class OutcryError(Exception):
    """Error during Outcry validation"""
    pass


class OutcryMissingKeyError(OutcryError):
    pass


class OutcryForbiddenKeyError(OutcryError):
    pass


class OutcryUnexpectedTypeError(OutcryError):
    pass


class OutcryValueError(OutcryError):
    pass



import json
from outcry.exceptions import OutcryError, OutcryForbiddenKeyError, OutcryMissingKeyError


class Representer(object):
    def __init__(self, obj):
        self._obj = obj
        self._validated = False
        self.validate()

    @property
    def schema(self):
        return {k: v for k, v in self.__class__.__dict__.items() if not k.startswith('_')}

    @property
    def validated(self):
        return self._validated
    
    def validate(self):
        if not self._has_invalid_keys and self._validations_pass:
            self._validated = True

    @property
    def _has_invalid_keys(self):
        if len(self._invalid_keys) > 0:
            raise OutcryForbiddenKeyError("Keys are notdefined in schema {0}".format(self._invalid_keys)) 
        if len(self._required_keys) > 0:
            raise OutcryMissingKeyError("The following keys are required {0}".format(self._required_keys))
        return False

    def _get_matching_value(self, name):
        return getattr(self._obj, name, None)

    @property
    def _validations_pass(self):
        return all(f.validate(self._get_matching_value(p)) for p, f in self.schema.items())

    @property
    def _invalid_keys(self):
        """Check if obj has any non-declared attributes"""
        return self._obj.__dict__.keys() - self.schema.keys()
    
    @property
    def _required_keys(self):
        required_keys = [k for k, f in self.schema.items() if f.required]
        return required_keys - self._obj.__dict__.keys()

    @property
    def _missing_keys(self):
        """Check if obj is missing any required keys"""
        return self._required_keys - self.schema.keys()
        
    @property
    def to_json(self):
        if not self._validated:
            raise OutcryError("Outcry: Validation error, cannot convert object to JSON")
        return json.dumps(self._obj, default=lambda o: o.__dict__, sort_keys=True)

    def from_json(self, data):
        return self._obj(**data)
 



import re
import math
from outcry.exceptions import OutcryValueError


class Validator(object):
    error = "Default validator error message"
    def __init__(self, fn, error):
        self._fn = fn
        self._error = error

    @property
    def error(self):
        return self._error

    def validate(self, value):
        valid = self._fn(value)
        if not valid:
            raise OutcryValueError("{0}, {1}".format(self._error, value)) 
        return True
        

def isStr():
    return Validator(lambda x: type(x) is str, "Value is not a string: ")


def isInt():
    return Validator(lambda x: type(x) is int, "Value is not a int: ")


def isList():
    return Validator(lambda y: type(y) is list, "Value is not a list: ")


def isEmail():
    pattern = re.compile("(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)")
    return Validator(pattern.match, "Email is not valid:")


def isUrl():
    pattern = re.compile("http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+")
    return Validator(pattern.match, "Url is not valid: ")


def isLength(minimum=0, maximum=math.inf):
    return Validator(lambda x: len(x) > minimum and len(x) < maximum)



class Property(object):
    def __init__(self, validators=None, required=False):
        self._validators = validators
        self._required = required

    def validate(self, value):
        if not self._validators:
            return True
        return all(v().validate(value) for v in self._validators)

    @property
    def required(self):
        return self._required
       

class Collection(Property):
    pass


class Field(Property):
    pass


def field(validators=None, required=False):
    return Field(validators=validators, required=required)




from distutils.core import setup

setup(
        name = 'outcry',
        packages = ['outcry'],
        version = '0.1.0',
        description = 'Parse and render REST documents using representers',
        author = 'Kareem Kwong',
        author_email = 'kareem.kwong@gmail.com',
        url = 'https://github.com/kwngo/outcry',
        download_url = 'https://github.com/kwngo/outcry/archive/0.1.0.tar.gz',
        keywords = ['json', 'parsing', 'render'],
        classifiers = []
        )
